import 'package:flutter/material.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:flutter_flicker/configuration/configuration.dart';
import 'decorator_utils.dart';

class FlickerDropDown<T> extends StatelessWidget {
  FlickerDropDown({
    Key? key,
    required this.labelText,
    this.uppercaseLabelAndHint = true,
    this.initialSelection,
    this.items,
    required this.onChanged,
    this.validator,
    this.optionTitles,
    this.readOnly = false,
    this.hintText,
  }) : super(key: key);

  final List<T>? items;
  final T? initialSelection;
  final String labelText;
  final bool uppercaseLabelAndHint;
  final ValueChanged<T?> onChanged;
  final Function? validator;
  final List<String>? optionTitles;
  final bool readOnly;
  final String? hintText;

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 250,
      margin: const EdgeInsets.only(
        top: 3.0,
        right: 8.0,
      ),
      alignment: Alignment.center,
      child: DropdownButtonFormField<T>(
        items: buildItemList(),
        value: initialSelection,
        onChanged: readOnly ? null : onChanged,
        validator: validator as String? Function(T?)?,
        isExpanded: true,
        decoration: InputDecoration(
          contentPadding: EdgeInsets.only(top: 25, bottom: 10),
          enabledBorder: UnderlineInputBorder(
            borderSide: BorderSide(
              color: readOnly
                  ? Colors.transparent
                  : FlickerConfiguration.colors.inputControlEnabledBorder,
              width: 0.2,
            ),
          ),
          disabledBorder: UnderlineInputBorder(
            borderSide: BorderSide(
              color: Colors.transparent,
              width: 0.2,
            ),
          ),
          focusedBorder: UnderlineInputBorder(
            borderSide: BorderSide(
              color: readOnly
                  ? Colors.transparent
                  : FlickerConfiguration.colors.inputControlFocusedBorder,
            ),
          ),
          labelText: DecoratorUtils.buildLabelText(labelText, uppercase: uppercaseLabelAndHint),
          floatingLabelBehavior: FloatingLabelBehavior.always,
          hintText: DecoratorUtils.buildSelectHintText(hintText, labelText: labelText, uppercase: uppercaseLabelAndHint),
          hintStyle: GoogleFonts.montserrat(
            fontSize: 15,
            color: FlickerConfiguration.colors.hintText,
            height: 1.0,
          ),
          labelStyle: GoogleFonts.montserrat(
            fontSize: 18,
            fontWeight: FontWeight.w600,
            color: readOnly
                ? Colors.grey.shade700
                : FlickerConfiguration.colors.inputLabel,
            height: 0.5,
          ),
          errorStyle: GoogleFonts.montserrat(
            fontSize: 12,
          ),
        ),
        style: GoogleFonts.montserrat(
          fontSize: 15,
          color: FlickerConfiguration.colors.input,
          height: 1.0,
        ),
      ),
    );
  }

  List<DropdownMenuItem<T>> buildItemList() {
    var itemList = <DropdownMenuItem<T>>[];
    for (var i = 0; i < items!.length; i++) {
      itemList.add(DropdownMenuItem<T>(
        child: Text(
            optionTitles == null ? items![i].toString() : optionTitles![i]),
        value: items![i],
      ));
    }
    return itemList;
  }
}
