import 'package:flutter/material.dart';

class FlickerGrid extends StatelessWidget {
  final List<DataColumn> columns;
  final List<DataRow> rows;
  final DataRow? footerRow;
  final double horizontalMargin;
  final bool showCheckboxColumn;

  const FlickerGrid({
    Key? key,
    this.columns = const [],
    this.rows = const [],
    this.footerRow,
    this.horizontalMargin = 0,
    this.showCheckboxColumn = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (isCardLayout(context)) {
      return cardLayout(context);
    } else {
      return gridLayout(context);
    }
  }

  bool isCardLayout(BuildContext context) {
    var width = MediaQuery.of(context).size.width;

    return width < 600;
  }

  Widget cardLayout(BuildContext context) {
    //add footer row for totals
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: rows
          .map(
            (row) => renderCard(row),
          )
          .toList(),
    );
  }

  Widget gridLayout(BuildContext context) {
    if (footerRow != null) {
      rows.add(footerRow!);
    }
    return DataTable(
      showCheckboxColumn: showCheckboxColumn,
      horizontalMargin: horizontalMargin,
      columns: columns,
      rows: rows,
    );
  }

  Card renderCard(DataRow row) {
    return Card(
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(15.0),
      ),
      elevation: 3,
      margin: EdgeInsets.all(
        4.0,
      ),
      child: InkWell(
        onTap: () {
          row.onSelectChanged!(true);
        },
        child: Padding(
          padding: const EdgeInsets.only(
            left: 8.0,
            right: 8.0,
            top: 16.0,
            bottom: 16.0,
          ),
          child: Column(
            children: row.cells
                .asMap()
                .entries
                .map(
                  (cell) => Padding(
                    padding: const EdgeInsets.all(4.0),
                    child: Row(
                      mainAxisSize: MainAxisSize.max,
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        columns[cell.key].label,
                        cell.value.child,
                      ],
                    ),
                  ),
                )
                .toList(),
          ),
        ),
      ),
    );
  }
}
