import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:flutter_flicker/controls/form_input/number_input.dart';
import 'package:flutter_flicker/configuration/configuration.dart';
import 'package:flutter_flicker/theme/colors.dart';

// todo: <extract shared test setup (this was copied from dfp)
typedef TestWidgetBuilder = Widget Function();

Widget createAppToWrapTestWidget(TestWidgetBuilder builder) => ProviderScope(
  child: MaterialApp(
    home: Scaffold(
      body: builder(),
    ),
  )
);

void setScreenSize(WidgetTester tester, {double w = 1600, double h = 1280}) {
  tester.binding.window.physicalSizeTestValue = Size(w, h);
  tester.binding.window.devicePixelRatioTestValue = 1.0;
}

// todo: extract/>

void  main() {
  FlickerConfiguration.set(
    colors: FlickerColors(),
  );

  group('FlickerNumberInput', () {
    testWidgets('loads correctly with default (minimal) parameters', (WidgetTester tester) async {
      setScreenSize(tester);
      await tester.pumpWidget(createAppToWrapTestWidget(() => FlickerNumberInput(
        initialValue: 12,
      )));

      expect(find.text('12'), findsOneWidget);
    });

    testWidgets('displays label and hint texts (uppercase by default)', (WidgetTester tester) async {
      setScreenSize(tester);
      await tester.pumpWidget(createAppToWrapTestWidget(() => FlickerNumberInput(
        initialValue: 12,
        labelText: 'label',
      )));

      expect(find.text('LABEL'), findsNWidgets(2), reason: 'expected to find label and hint, uppercase');
    });

    group('prefix and suffix', () {
      testWidgets('display the prefix and suffix widgets', (WidgetTester tester) async {
        setScreenSize(tester);
        await tester.pumpWidget(createAppToWrapTestWidget(() => FlickerNumberInput(
          initialValue: 12,
          prefix: Text('pre'),
          suffix: Text('post'),
        )));

        expect(find.text('pre'), findsOneWidget);
        expect(find.text('post'), findsOneWidget);
      });

      testWidgets('displayed with uppercase labels', (WidgetTester tester) async {
        setScreenSize(tester);
        await tester.pumpWidget(createAppToWrapTestWidget(() => FlickerNumberInput(
          initialValue: 12,
          labelText: 'Label', // implicit uppercase
          prefix: Text('pre'),
          suffix: Text('post'),
        )));

        expect(find.text('LABEL'), findsNWidgets(2), reason: 'expected to find label and hint, uppercase');
        expect(find.text('pre'), findsOneWidget);
        expect(find.text('post'), findsOneWidget);
      });

      testWidgets('displayed with uppercase labels', (WidgetTester tester) async {
        setScreenSize(tester);
        await tester.pumpWidget(createAppToWrapTestWidget(() => FlickerNumberInput(
          initialValue: 12,
          labelText: 'Label',
          uppercaseLabelAndHint: false, // explicit lowercase
          prefix: Text('pre'),
          suffix: Text('post'),
        )));

        expect(find.text('Label'), findsNWidgets(2), reason: 'expected to find label and hint, as-is');
        expect(find.text('pre'), findsOneWidget);
        expect(find.text('post'), findsOneWidget);
      });
    });
  });
}
