import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:flutter_flicker/navigation/route_configuration.dart';
import 'package:flutter_flicker/configuration/configuration.dart';
import 'package:flutter_test/flutter_test.dart';

void  main() {
  FlickerConfiguration.routes = [ /* need only be initialized*/ ];

  group('parameters', () {
    test('empty string', () {
      final current = CurrentRouteConfiguration();
      current.updateConfiguration('route', '', (_) {}, false);
      expect(current.parameter, '');
    });

    test('null', () {
      final current = CurrentRouteConfiguration();
      current.updateConfiguration('route', '', (_) {}, false);
      expect(current.parameter, '', reason: 'should be empty string, not null');
    });

    group('parses', () {
      test('single item values (use for example as ids)', () {
        final current = CurrentRouteConfiguration();
        current.updateConfiguration('route', '123', (_) {}, false);
        expect(current.parameter, '123');
      });

      test('empty query-string like parameters', () {
        final current = CurrentRouteConfiguration();

        current.updateConfiguration('route', null, (_) {}, false);
        expect(current.queryParameters.isEmpty, true, reason: 'expect null to be empty query parameters');

        current.updateConfiguration('route', '', (_) {}, false);
        expect(current.queryParameters.isEmpty, true, reason: 'expect empty-string to be empty query parameters');

        current.updateConfiguration('route', '123', (_) {}, false);
        expect(current.queryParameters.isEmpty, true, reason: 'expect id-style parameter to be empty query parameters');
      });

      test('multiple query-string like parameters', () {
        final current = CurrentRouteConfiguration();

        current.updateConfiguration('route', '?name=john', (_) {}, false);
        expect(current.queryParameters.isEmpty, false);
        expect(current.queryParameters['name'], 'john');
        expect(current.queryParameters['surname'], null);

        current.updateConfiguration('route', '?name=john&surname=doe', (_) {}, false);
        expect(current.queryParameters.isEmpty, false);
        expect(current.queryParameters['name'], 'john');
        expect(current.queryParameters['surname'], 'doe');
      });
    });

    group('build query string parameters', () {
      test('empty for empty hash', () {
        final current = CurrentRouteConfiguration();

        var didCall = false;
        current.updateConfiguration('route', '', (r) {
          expect(r.parameter, '');
          didCall = true;
        }, false);

        final route = RouteInfo(
          routeName: 'route',
          screen: () => throw 'not implemented',
          logicProvider: ChangeNotifierProvider.autoDispose((ref) => throw 'not implemented'),
        );
        current.navigateWithQuery(route, {});

        expect(didCall, true, reason: 'did not call the requestRoute param from updateConfiguration');
      });

      test('built from hash', () {
        final current = CurrentRouteConfiguration();

        var didCall = false;
        current.updateConfiguration('route', '', (r) {
          expect(r.parameter, '?foo=bar');
          didCall = true;
        }, false);

        final route = RouteInfo(
          routeName: 'route',
          screen: () => throw 'not implemented',
          logicProvider: ChangeNotifierProvider.autoDispose((ref) => throw 'not implemented'),
        );
        current.navigateWithQuery(route, { 'foo': 'bar' });

        expect(didCall, true, reason: 'did not call the requestRoute param from updateConfiguration');
      });
    });
  });
}
